@@ -15,17 +15,19 @@
 #
 # @author: Stéphane Albert
 #
-from cloudkitty import orchestrator
 from cloudkitty import service
 
 
 def main():
     service.prepare_service()
-    processor = orchestrator.Orchestrator()
-    try:
-        processor.process()
-    except KeyboardInterrupt:
-        processor.terminate()
+
+    # NOTE(mc): This import is done here to ensure that the prepare_service()
+    # function is called before any cfg option. By importing the orchestrator
+    # file, the utils one is imported too, and then some cfg options are read
+    # before the prepare_service(), making cfg.CONF returning default values
+    # systematically.
+    from cloudkitty import orchestrator
+    orchestrator.OrchestratorServiceManager().run()
 
 
 if __name__ == '__main__':
