@@ -36,6 +36,9 @@ LOG = logging.getLogger(__name__)
 MONASCA_API_VERSION = '2_0'
 COLLECTOR_MONASCA_OPTS = 'collector_monasca'
 
+keystone_opts = ks_loading.get_auth_common_conf_options() + \
+    ks_loading.get_session_conf_options()
+
 collector_monasca_opts = [
     cfg.StrOpt(
         'interface',
@@ -49,14 +52,9 @@ collector_monasca_opts = [
     ),
 ]
 
-cfg.CONF.register_opts(collector_monasca_opts, COLLECTOR_MONASCA_OPTS)
-ks_loading.register_session_conf_options(
-    cfg.CONF,
-    COLLECTOR_MONASCA_OPTS)
-ks_loading.register_auth_conf_options(
-    cfg.CONF,
-    COLLECTOR_MONASCA_OPTS)
 CONF = cfg.CONF
+CONF.register_opts(keystone_opts, COLLECTOR_MONASCA_OPTS)
+CONF.register_opts(collector_monasca_opts, COLLECTOR_MONASCA_OPTS)
 
 METRICS_CONF = ck_utils.load_conf(CONF.collect.metrics_conf)
 
@@ -69,13 +67,15 @@ MONASCA_EXTRA_SCHEMA = {
             All(str, Length(min=1)),
         Required('aggregation_method', default='max'):
             In(['max', 'mean', 'min']),
+        # In case the metrics in Monasca do not belong to the project
+        # cloudkitty is identified in
+        Required('forced_project_id', default=''): str,
     },
 }
 
 
 class EndpointNotFound(Exception):
     """Exception raised if the Monasca endpoint is not found"""
-    pass
 
 
 class MonascaCollector(collector.BaseCollector):
@@ -106,10 +106,7 @@ class MonascaCollector(collector.BaseCollector):
             CONF,
             COLLECTOR_MONASCA_OPTS,
             auth=self.auth)
-        self.ks_client = ks_client.Client(
-            session=self.session,
-            interface=CONF.collector_monasca.interface,
-        )
+        self.ks_client = ks_client.Client(session=self.session)
         self.mon_endpoint = self._get_monasca_endpoint()
         if not self.mon_endpoint:
             raise EndpointNotFound()
@@ -183,6 +180,10 @@ class MonascaCollector(collector.BaseCollector):
         period = end - start
 
         extra_args = self.conf[metric_name]['extra_args']
+        kwargs = {}
+        if extra_args['forced_project_id']:
+            kwargs['tenant_id'] = extra_args['forced_project_id']
+
         return self._conn.metrics.list_statistics(
             name=metric_name,
             merge_metrics=True,
@@ -191,7 +192,8 @@ class MonascaCollector(collector.BaseCollector):
             end_time=ck_utils.ts2dt(end),
             period=period,
             statistics=extra_args['aggregation_method'],
-            group_by=group_by)
+            group_by=group_by,
+            **kwargs)
 
     def _fetch_metrics(self, metric_name, start, end,
                        project_id=None, q_filter=None):
