@@ -17,6 +17,7 @@
 #
 import decimal
 
+from keystoneauth1 import session as ks_sess
 import mock
 from oslo_config import fixture as config_fixture
 from oslotest import base
@@ -71,8 +72,7 @@ class TestCase(testscenarios.TestWithScenarios, base.BaseTestCase):
         super(TestCase, self).setUp()
         self._conf_fixture = self.useFixture(config_fixture.Config())
         self.conf = self._conf_fixture.conf
-        self.conf.set_override('connection', self.db_url, 'database',
-                               enforce_type=True)
+        self.conf.set_override('connection', self.db_url, 'database')
         self.conn = ck_db_api.get_instance()
         migration = self.conn.get_migration()
         migration.upgrade('head')
@@ -83,7 +83,7 @@ class TestCase(testscenarios.TestWithScenarios, base.BaseTestCase):
         self.auth = auth
         session = mock.patch(
             'keystoneauth1.loading.load_session_from_conf_options',
-            return_value=dict())
+            return_value=ks_sess.Session())
         session.start()
         self.session = session
 
