@@ -13,9 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-# @author: Stéphane Albert
-#
 import abc
+import datetime
 import decimal
 import os
 
@@ -24,25 +23,31 @@ import mock
 from oslo_config import cfg
 from oslo_config import fixture as conf_fixture
 from oslo_db.sqlalchemy import utils
-import oslo_messaging as messaging
+import oslo_messaging
 from oslo_messaging import conffixture
 from oslo_policy import opts as policy_opts
 import six
 from stevedore import driver
 from stevedore import extension
+import webob.dec
 from wsme import types as wtypes
 import wsmeext.pecan as wsme_pecan
 
 from cloudkitty.api import app
-from cloudkitty.common import rpc
+from cloudkitty.api import middleware
 from cloudkitty import db
 from cloudkitty.db import api as ck_db_api
+from cloudkitty import messaging
 from cloudkitty import rating
 from cloudkitty import storage
-from cloudkitty.storage.sqlalchemy import models
+from cloudkitty.storage.v1.sqlalchemy import models
+from cloudkitty import storage_state
 from cloudkitty import tests
+from cloudkitty.tests.storage.v2 import influx_utils
+from cloudkitty.tests import utils as test_utils
 from cloudkitty import utils as ck_utils
 
+
 INITIAL_TIMESTAMP = 1420070400
 
 
@@ -179,27 +184,27 @@ class RatingModulesFixture(BaseExtensionFixture):
 
 
 class ConfigFixture(fixture.GabbiFixture):
+    auth_strategy = 'noauth'
+
     def start_fixture(self):
         self.conf = None
         conf = conf_fixture.Config().conf
         policy_opts.set_defaults(conf)
         msg_conf = conffixture.ConfFixture(conf)
-        msg_conf.transport_driver = 'fake'
+        msg_conf.transport_url = 'fake:/'
         conf.import_group('api', 'cloudkitty.api.app')
-        conf.set_override('auth_strategy', 'noauth', enforce_type=True)
-        conf.set_override('connection', 'sqlite:///', 'database',
-                          enforce_type=True)
+        conf.set_override('auth_strategy', self.auth_strategy)
+        conf.set_override('connection', 'sqlite:///', 'database')
         conf.set_override('policy_file',
                           os.path.abspath('etc/cloudkitty/policy.json'),
-                          group='oslo_policy',
-                          enforce_type=True)
+                          group='oslo_policy')
         conf.set_override('api_paste_config',
                           os.path.abspath(
                               'cloudkitty/tests/gabbi/gabbi_paste.ini')
                           )
         conf.import_group('storage', 'cloudkitty.storage')
-        conf.set_override('backend', 'sqlalchemy', 'storage',
-                          enforce_type=True)
+        conf.set_override('backend', 'sqlalchemy', 'storage')
+        conf.set_override('version', '1', 'storage')
         self.conf = conf
         self.conn = ck_db_api.get_instance()
         migration = self.conn.get_migration()
@@ -211,28 +216,75 @@ class ConfigFixture(fixture.GabbiFixture):
         db.get_engine().dispose()
 
 
+class ConfigFixtureStorageV2(ConfigFixture):
+
+    def start_fixture(self):
+        super(ConfigFixtureStorageV2, self).start_fixture()
+        self.conf.set_override('backend', 'influxdb', 'storage')
+        self.conf.set_override('version', '2', 'storage')
+
+
+class ConfigFixtureKeystoneAuth(ConfigFixture):
+    auth_strategy = 'keystone'
+
+    def start_fixture(self):
+        # Mocking the middleware process_request which check for credentials
+        # here, the only check done is that the hardcoded token is the one
+        # send by the query. If not, 401, else 200.
+        def _mock_proc_request(self, request):
+            token = 'c93e3e31342e4e32ba201fd3d70878b5'
+            http_code = 401
+            if 'X-Auth-Token' in request.headers and \
+               request.headers['X-Auth-Token'] == token:
+                http_code = 200
+
+            return webob.Response(
+                status_code=http_code,
+                content_type='application/json'
+            )
+
+        self._orig_func = middleware.auth_token.AuthProtocol.process_request
+        middleware.auth_token.AuthProtocol.process_request = _mock_proc_request
+
+        super(ConfigFixtureKeystoneAuth, self).start_fixture()
+
+    def stop_fixture(self):
+        super(ConfigFixtureKeystoneAuth, self).stop_fixture()
+        middleware.auth_token.AuthProtocol.process_request = self._orig_func
+
+
 class BaseFakeRPC(fixture.GabbiFixture):
     endpoint = None
 
     def start_fixture(self):
-        rpc.init()
-        target = messaging.Target(topic='cloudkitty',
-                                  server=cfg.CONF.host,
-                                  version='1.0')
+        messaging.setup()
+        target = oslo_messaging.Target(topic='cloudkitty',
+                                       server=cfg.CONF.host,
+                                       version='1.0')
         endpoints = [
             self.endpoint()
         ]
-        self.server = rpc.get_server(target, endpoints)
+        self.server = messaging.get_server(target, endpoints)
         self.server.start()
 
     def stop_fixture(self):
         self.server.stop()
 
 
+class ScopeStateResetFakeRPC(BaseFakeRPC):
+    class FakeRPCEndpoint(object):
+        target = oslo_messaging.Target(version='1.0')
+
+        def reset_state(self, ctxt, res_data):
+            pass
+
+    endpoint = FakeRPCEndpoint
+
+
 class QuoteFakeRPC(BaseFakeRPC):
     class FakeRPCEndpoint(object):
-        target = messaging.Target(namespace='rating',
-                                  version='1.0')
+        target = oslo_messaging.Target(namespace='rating',
+                                       version='1.0')
 
         def quote(self, ctxt, res_data):
             return str(1.0)
@@ -241,17 +293,18 @@ class QuoteFakeRPC(BaseFakeRPC):
 
 
 class BaseStorageDataFixture(fixture.GabbiFixture):
-    def create_fake_data(self, begin, end):
+    def create_fake_data(self, begin, end, project_id):
         data = [{
             "period": {
                 "begin": begin,
                 "end": end},
             "usage": {
-                "compute": [
+                "cpu": [
                     {
                         "desc": {
                             "dummy": True,
-                            "fake_meta": 1.0},
+                            "fake_meta": 1.0,
+                            "project_id": project_id},
                         "vol": {
                             "qty": 1,
                             "unit": "nothing"},
@@ -261,11 +314,12 @@ class BaseStorageDataFixture(fixture.GabbiFixture):
                 "begin": begin,
                 "end": end},
             "usage": {
-                "image": [
+                "image.size": [
                     {
                         "desc": {
                             "dummy": True,
-                            "fake_meta": 1.0},
+                            "fake_meta": 1.0,
+                            "project_id": project_id},
                         "vol": {
                             "qty": 1,
                             "unit": "nothing"},
@@ -282,7 +336,7 @@ class BaseStorageDataFixture(fixture.GabbiFixture):
             return_value=dict())
         with auth:
             with session:
-                self.storage = storage.get_storage()
+                self.storage = storage.get_storage(conf=test_utils.load_conf())
         self.storage.init()
         self.initialize_data()
 
@@ -300,28 +354,19 @@ class StorageDataFixture(BaseStorageDataFixture):
         nodata_duration = (24 * 3 + 12) * 3600
         tenant_list = ['8f82cc70-e50c-466e-8624-24bdea811375',
                        '7606a24a-b8ad-4ae0-be6c-3d7a41334a2e']
-        for tenant in tenant_list:
-            for i in range(INITIAL_TIMESTAMP,
-                           INITIAL_TIMESTAMP + nodata_duration,
-                           3600):
-                self.storage.nodata(i, i + 3600, tenant)
         data_ts = INITIAL_TIMESTAMP + nodata_duration + 3600
         data_duration = (24 * 2 + 8) * 3600
         for i in range(data_ts,
                        data_ts + data_duration,
                        3600):
-            data = self.create_fake_data(i, i + 3600)
-            self.storage.append(data, tenant_list[0])
+            data = self.create_fake_data(i, i + 3600, tenant_list[0])
+            self.storage.push(data, tenant_list[0])
         half_duration = int(data_duration / 2)
         for i in range(data_ts,
                        data_ts + half_duration,
                        3600):
-            data = self.create_fake_data(i, i + 3600)
-            self.storage.append(data, tenant_list[1])
-        for i in range(data_ts + half_duration + 3600,
-                       data_ts + data_duration,
-                       3600):
-            self.storage.nodata(i, i + 3600, tenant_list[1])
+            data = self.create_fake_data(i, i + 3600, tenant_list[1])
+            self.storage.push(data, tenant_list[1])
 
 
 class NowStorageDataFixture(BaseStorageDataFixture):
@@ -330,9 +375,36 @@ class NowStorageDataFixture(BaseStorageDataFixture):
         for i in range(begin,
                        begin + 3600 * 12,
                        3600):
-            data = self.create_fake_data(i, i + 3600)
-            self.storage.append(data,
-                                '3d9a1b33-482f-42fd-aef9-b575a3da9369')
+            project_id = '3d9a1b33-482f-42fd-aef9-b575a3da9369'
+            data = self.create_fake_data(i, i + 3600, project_id)
+            self.storage.push(data, project_id)
+
+
+class ScopeStateFixture(fixture.GabbiFixture):
+
+    def start_fixture(self):
+        self.sm = storage_state.StateManager()
+        self.sm.init()
+        data = [
+            ('aaaa', datetime.datetime(2019, 1, 1), 'fet1', 'col1', 'key1'),
+            ('bbbb', datetime.datetime(2019, 2, 2), 'fet1', 'col1', 'key2'),
+            ('cccc', datetime.datetime(2019, 3, 3), 'fet1', 'col2', 'key1'),
+            ('dddd', datetime.datetime(2019, 4, 4), 'fet1', 'col2', 'key2'),
+            ('eeee', datetime.datetime(2019, 5, 5), 'fet2', 'col1', 'key1'),
+            ('ffff', datetime.datetime(2019, 6, 6), 'fet2', 'col1', 'key2'),
+            ('gggg', datetime.datetime(2019, 6, 6), 'fet2', 'col2', 'key1'),
+            ('hhhh', datetime.datetime(2019, 6, 6), 'fet2', 'col2', 'key2'),
+        ]
+        for d in data:
+            self.sm.set_state(
+                d[0], d[1], fetcher=d[2], collector=d[3], scope_key=d[4])
+
+    def stop_fixture(self):
+        session = db.get_session()
+        q = utils.model_query(
+            self.sm.model,
+            session)
+        q.delete()
 
 
 class CORSConfigFixture(fixture.GabbiFixture):
@@ -356,8 +428,48 @@ class CORSConfigFixture(fixture.GabbiFixture):
         cfg.ConfigOpts.GroupAttr.__getattr__ = self._original_call_method
 
 
+class MetricsConfFixture(fixture.GabbiFixture):
+    """Inject Metrics configuration mock to the get_metrics_conf() function"""
+
+    def start_fixture(self):
+        self._original_function = ck_utils.load_conf
+        ck_utils.load_conf = mock.Mock(
+            return_value=tests.samples.METRICS_CONF,
+        )
+
+    def stop_fixture(self):
+        """Remove the get_metrics_conf() monkeypatch."""
+        ck_utils.load_conf = self._original_function
+
+
+class InfluxStorageDataFixture(NowStorageDataFixture):
+
+    def start_fixture(self):
+        cli = influx_utils.FakeInfluxClient()
+        st = storage.get_storage()
+        st._conn = cli
+
+        self._get_storage_patch = mock.patch(
+            'cloudkitty.storage.get_storage',
+            new=lambda **kw: st,
+        )
+        self._get_storage_patch.start()
+
+        super(InfluxStorageDataFixture, self).start_fixture()
+
+    def initialize_data(self):
+        data = test_utils.generate_v2_storage_data(
+            start=ck_utils.get_month_start(),
+            end=ck_utils.utcnow().replace(hour=0),
+        )
+        self.storage.push([data])
+
+    def stop_fixture(self):
+        self._get_storage_patch.stop()
+
+
 def setup_app():
-    rpc.init()
+    messaging.setup()
     # FIXME(sheeprine): Extension fixtures are interacting with transformers
     # loading, since collectors are not needed here we shunt them
     no_collector = mock.patch(
